import { Component } from '@angular/core';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {
  data: any[] = [];

  columns: any[] = [
    { field: 'string', dataType: 'string' },
    { field: 'number', dataType: 'number' },
    { field: 'date', dataType: 'date' },
    { field: 'boolean', dataType: 'boolean' },
  ];

  handleFiles(files: FileList): void {
    // https://developer.mozilla.org/ja/docs/Web/API/File/Using_files_from_web_applications
    if (files.length !== 1) {
      alert('複数ファイルは読み込めません');
      return;
    }
    this.importCsv(files[0]);
  }

  private importCsv(file: File): void {
    const reader = new FileReader();
    // https://developer.mozilla.org/ja/docs/Web/API/FileReader/onload
    reader.onload = () => {
      // https://developer.mozilla.org/ja/docs/Web/API/FileReader/result#%E4%BE%8B
      // CSV -> 配列に変換して igx-grid にバインド
      this.data = this.csvToArray(reader.result as string);
    };
    // https://developer.mozilla.org/ja/docs/Web/API/FileReader/onerror
    reader.onerror = () => {
      reader.abort();
    };
    // https://developer.mozilla.org/ja/docs/Web/API/FileReader/readAsText
    reader.readAsText(file);
  }

  // CSVテキストを変換する方法は様々あり、以下の実装は一例にすぎず、完全なものではありません。
  // あらかじめご了承ください。
  private csvToArray(text: string): any[] {
    // 改行コードで行分割
    const rows = text.split(/\r\n|\n/);
    if (rows.length === 0) {
      return [];
    }

    // １行目はヘッダー行とみなして捨てる
    rows.shift();

    const data = [];
    for (const row of rows) {
      if (!row) {
        continue;
      }

      // タブコードでセル分割
      const cells = row.split('\t');

      const rowObj = {};
      for (const [index, cell] of cells.entries()) {
        // 列定義を取得
        const column = this.columns[index];
        let value;

        // データタイプに応じて簡易的な型変換を行う
        switch (column.dataType) {
          case 'number': {
            value = Number(cell);
            break;
          }
          case 'date': {
            value = new Date(cell);
            break;
          }
          case 'boolean': {
            value = cell.toLowerCase() === 'true';
            break;
          }
          default: {
            value = cell;
            break;
          }
        }
        // 列のフィールド名に応じた値をセット
        rowObj[column.field] = value;
      }

      data.push(rowObj);
    }

    return data;
  }
}
